!!****if* source/Simulation/SimulationMain/Sod/Simulation_initBlock
!!
!! NAME
!!
!!  Simulation_initBlock
!!
!!
!! SYNOPSIS
!!
!!  Simulation_initBlock(integer(IN) :: blockID, 
!!                       integer(IN) :: myPE)
!!
!!
!!
!! DESCRIPTION
!!
!!  Initializes fluid data (density, pressure, velocity, etc.) for
!!  a specified block.  This version uses particle sph data to set up
!!  the block.
!!
!!
!! ARGUMENTS
!!
!!  blockID -           the number of the block to update
!!  myPE -             my processor number
!!
!! PARAMETERS
!!    ???
!!
!! GLOBAL VARIABLES
!!  sim_npart
!!  sim_partPos(npart,3)
!!  sim_partVel(npart,3)
!!  sim_partH(npart)
!!  sim_partM(npart)
!!  sim_partU(npart)
!!
!!  sim_xar36
!!  sim_xc12
!!  sim_xca40
!!  sim_xcr48
!!  sim_xfe52
!!  sim_xhe4
!!  sim_xmg24
!!  sim_xne20
!!  sim_xni56
!!  sim_xo16
!!  sim_xs32
!!  sim_xsi28
!!  sim_xti44
!!
!! LOCAL VARIABLES
!!  pos(3)
!!  vel(3)
!!  h
!!  m
!!  f
!!  var(?)
!!
!!  xCen(sizeX)
!!  yCen(sizeY)
!!  zCen(sizeZ)
!!
!!  width(3)
!!
!!  inside
!!
!!  blkLow(3), blkHigh(3)
!! 
!!  rhoZone, velZone(3), presZone, eintZone, enerZone, ekinZone
!! 
!!***

subroutine Simulation_initBlock(blockID, myPE)

  use Simulation_data

  use Driver_interface, ONLY: Driver_abortFlash
 
  use Grid_interface, ONLY : Grid_getBlkIndexLimits, Grid_getCellCoords, &
                             Grid_getPointData, Grid_putPointData, &
                             Grid_getBlkPtr, Grid_releaseBlkPtr

  use Eos_interface, ONLY : Eos

!  use Multispecies_interface, ONLY : 

  use Logfile_interface, ONLY : Logfile_stamp

  implicit none

#include "constants.h"
#include "Flash.h"
#include "Eos.h"
#include "Multispecies.h"
  
  integer, intent(in) :: blockID
  integer, intent(in) :: myPE

  real :: w

  real, parameter :: Boltzmann = 1.3807e-16
! Dummy indices / values
  integer :: i, j, k, n
  real :: q1, q2, q3, q4

! Particular particle values
  real, dimension(MDIM) :: pos, vel
  real :: h, m, f, u, r, T, P

! Grid values
  real :: width
  real,allocatable, dimension(:) :: xCen, yCen, zCen
  integer :: sizeX,sizeY,sizeZ, deltaCell
  integer, dimension(2,MDIM) :: blkLimits, blkLimitsGC
  real, dimension(MDIM) :: blkLow, blkHigh

! Values to be put in cell  
  real :: rhoZone,  presZone, eintZone, enerZone, ekinZone, tempZone, gammZone
  real, dimension(MDIM) :: velZone
  integer, dimension(MDIM) :: posCell
  real, dimension(:,:,:,:),pointer :: solnData
  
! Variables for Eos
  real, dimension(EOS_NUM) :: eosData
  integer eosMode

! Variables for Multispecies
  real, dimension(SPECIES_BEGIN:SPECIES_END) :: massFraction
  real :: xar36, xc12, xca40, xcr48, xfe52, xhe4, xmg24, xne20, xni56, &
          xo16 , xs32, xsi28, xti44
 
  logical :: gcell = .false., inside, inrange
  integer :: istat 

  character(len=30) :: outputFile
  
  real :: smallNum = 1e-10

!!  ! dump some output to stdout listing the paramters
  if (myPE == MASTER_PE) then
          
1    format (1X, 1P, 4(A7, E13.7, :, 1X))
2    format (1X, 1P, 2(A7, E13.7, 1X), A7, I13)
!!     
  endif
  
  
  ! get the integer index information for the current block
  call Grid_getBlkIndexLimits(blockId,blkLimits,blkLimitsGC)
  
  ! Determine size of interior of block
  sizeX = blkLimits(HIGH,IAXIS) - blkLimits(LOW,IAXIS) + 1
  sizeY = blkLimits(HIGH,JAXIS) - blkLimits(LOW,JAXIS) + 1
  sizeZ = blkLimits(HIGH,KAXIS) - blkLimits(LOW,KAXIS) + 1

  ! allocate memory for iCen variables
  allocate(xCen(sizeX),stat=istat)
  allocate(yCen(sizeY),stat=istat)
  allocate(zCen(sizeZ),stat=istat)

  ! Initialize
  xCen = 0.0E0
  yCen = 0.0E0
  zCen = 0.0E0

  ! Get interior cell coordinates
  if (NDIM == 3) &
    call Grid_getCellCoords(KAXIS, blockId, CENTER ,gcell, zCen, sizeZ)

  if (NDIM >= 2) &
    call Grid_getCellCoords(JAXIS, blockId, CENTER,gcell, yCen, sizeY)

  call Grid_getCellCoords(IAXIS, blockId, CENTER,gcell, xCen, sizeX)
 
  ! Get lower and upper coordinates
  blkLow(1)   = xCen(1)
  blkHigh(1) = xCen(sizeX)

  blkLow(2)   = yCen(1)
  blkHigh(2) = yCen(sizeY)

  blkLow(3)   = zCen(1)
  blkHigh(3) = zCen(sizeZ)
 
  ! Get cell dimensions; currently required to be cubic
  width = xCen(2) - xCen(1)

  q4 = abs(width-(yCen(2)-yCen(1)))
  if((q4.gt.smallNum*width).and.(NDIM>1)) then
    write(6,*) "Non-symmetric cell dimensions"
    write(6,*) "Width = ", width, " while new y width = ",yCen(2)-yCen(1)
    call Driver_abortFlash('Non symmetric cell dimensions. See output file for more details')
  endif
 
  q4 = abs(width-(zCen(2)-zCen(1)))
  if((q4.gt.smallNum*width).and.(NDIM>2)) then
    write(6,*) "Non-symmetric cell dimension"
    write(6,*) "Width = ", width, " while new z width = ",zCen(2)-zCen(1) 
    call Driver_abortFlash('Non symmetric cell dimensions. See output file for more details')
  endif

  ! Get pointer to unk
  call Grid_getBlkPtr(blockId, solnData)

  ! Take note of some confusion:
  !   - Cell positions are ONLY found for the interior (not including GC)
  !   - solnData points to all cells INCLUDING GC
  !   - posCell will have to use the GC grid if it will later be used with solnData
  !      - thus for comparisons between posCell and iCen, iCen must have the same
  !        index as posCell minus blkLimits(DIRECTION) + 1

!------------------------------------------------------------------------------
! Loop over all particle positions. If particle's smoothing length is within the
! the block, then it's attributes are applied to its neighbouring cells, subject to
! the particle's smoothing kernel.

  do n=1,sim_npart

    inrange= .true.
    inside = .true.

    h = sim_partH(n)
    pos(:) = 0.e0

    ! Check that Particle's smoothing length overlaps with block
    do j=1,MDIM
      if (sim_partPos(n,j).lt.blkLow(j)-width-sqrt(4.E0*h*h-pos(1)-pos(2))) then
        inrange= .false.
        inside = .false.
      else if (sim_partPos(n,j).lt.blkLow(j)-width) then
        pos(j)=blkLow(j)-width-sim_partPos(n,j)
        pos(j)=pos(j)*pos(j)
        inside = .false.
      else if (sim_partPos(n,j).lt.blkHigh(j)+width) then
        pos(j)=0.e0
      else if (sim_partPos(n,j).lt.blkHigh(j)+width+sqrt(4.E0*h*h-pos(1)-pos(2))) then
        pos(j)=sim_partPos(n,j)-blkHigh(j)
        pos(j)=pos(j)*pos(j)
        inside = .false.
      else
        inrange= .false. 
        inside = .false.
      endif
    enddo

    if(inrange) then       ! Success: particle may affect a cell in this block

      ! Initialize particular particle data values
      do j=1,MDIM
        pos(j) = sim_partPos(n,j)
        vel(j) = sim_partVel(n,j)
        posCell(j) = blkLimits(LOW,j) + int(0.5E0 + (pos(j)-blkLow(j))/width)
      enddo
      m = sim_partM(n)
      u = sim_partU(n)
      T = sim_partT(n)
      P = sim_partP(n)

      if(n.eq.1) then
        write(6,*) "part x y z = ", pos(1), " , ", pos(2)," , ",  pos(3)
        write(6,*) "Cell x y z = ", posCell(1), " , ", posCell(2)," , ",  posCell(3)
        write(6,*) "mass = ", m
        write(6,*) "u = ", u, " ,T = ", T, " ,P = ", P
        write(6,*) "width = ", width
        write(6,*) "inside=", inside
      endif
 
      ! Determine Mass conservation Factor
      call findFact(pos(1),pos(2),pos(3),blkLow(1),blkLow(2),blkLow(3),h,f,width,deltaCell, BlockID, myPE)

      if(n.eq.1) write(6,*) "factor and delta = ", f, " , ", deltaCell

      if ((f .eq. 0.e0).and. inside) then ! smoothing length doesn't touch even the center of the cell
                                       ! it is inside: treat as if complete particle is in cell.
           write(6,*) "Particle lies completely inside this current box:"
           i = posCell(IAXIS)
           j = posCell(JAXIS)
           k = posCell(KAXIS)
           write(6,*) i, ",", j,",", k
           rhoZone  = m / width**3

           velZone(IAXIS) = vel(IAXIS)*rhoZone
           velZone(JAXIS) = vel(JAXIS)*rhoZone
           velZone(KAXIS) = vel(KAXIS)*rhoZone

           eintZone = u*rhoZone
           tempZone = T*rhoZone
           presZone = P*rhoZone

           solnData(DENS_VAR,i,j,k) = solnData(DENS_VAR,i,j,k) + rhoZone     
           solnData(VELX_VAR,i,j,k) = solnData(VELX_VAR,i,j,k) + velZone(IAXIS)     
           solnData(VELY_VAR,i,j,k) = solnData(VELY_VAR,i,j,k) + velZone(JAXIS)     
           solnData(VELZ_VAR,i,j,k) = solnData(VELZ_VAR,i,j,k) + velZone(KAXIS)     
           solnData(EINT_VAR,i,j,k) = solnData(EINT_VAR,i,j,k) + eintZone
           solnData(TEMP_VAR,i,j,k) = solnData(TEMP_VAR,i,j,k) + tempZone
           solnData(PRES_VAR,i,j,k) = solnData(PRES_VAR,i,j,k) + presZone

           xar36 = sim_xar36(n)*rhozone
           xc12  = sim_xc12(n)*rhozone
           xca40 = sim_xca40(n)*rhozone
           xcr48 = sim_xcr48(n)*rhozone
           xfe52 = sim_xfe52(n)*rhozone
           xhe4  = sim_xhe4(n)*rhozone
           xmg24 = sim_xmg24(n)*rhozone
           xne20 = sim_xne20(n)*rhozone
           xni56 = sim_xni56(n)*rhozone
           xo16  = sim_xo16(n)*rhozone
           xs32  = sim_xs32(n)*rhozone
           xsi28 = sim_xsi28(n)*rhozone
           xti44 = sim_xti44(n)*rhozone

           solnData(AR36_SPEC,i,j,k) = solnData(AR36_SPEC,i,j,k) + xar36
           solnData( C12_SPEC,i,j,k) = solnData( C12_SPEC,i,j,k) + xc12
           solnData(CA40_SPEC,i,j,k) = solnData(CA40_SPEC,i,j,k) + xca40
           solnData(CR48_SPEC,i,j,k) = solnData(CR48_SPEC,i,j,k) + xcr48
           solnData(FE52_SPEC,i,j,k) = solnData(FE52_SPEC,i,j,k) + xfe52
           solnData( HE4_SPEC,i,j,k) = solnData( HE4_SPEC,i,j,k) + xhe4
           solnData(MG24_SPEC,i,j,k) = solnData(MG24_SPEC,i,j,k) + xmg24
           solnData(NE20_SPEC,i,j,k) = solnData(NE20_SPEC,i,j,k) + xne20
           solnData(NI56_SPEC,i,j,k) = solnData(NI56_SPEC,i,j,k) + xni56
           solnData( O16_SPEC,i,j,k) = solnData( O16_SPEC,i,j,k) + xo16
           solnData( S32_SPEC,i,j,k) = solnData( S32_SPEC,i,j,k) + xs32
           solnData(SI28_SPEC,i,j,k) = solnData(SI28_SPEC,i,j,k) + xsi28
           solnData(TI44_SPEC,i,j,k) = solnData(TI44_SPEC,i,j,k) + xti44
           
           !put in value of default species
!           if (NSPECIES > 0) then
!              call Grid_putPointData(blockID, CENTER, SPECIES_BEGIN, EXTERIOR, &
!                   axis, 1.0e0-(NSPECIES-1)*sim_smallX)
!
!
!              !if there is only 1 species, this loop will not execute
!              do n = SPECIES_BEGIN+1,SPECIES_END
!                 call Grid_putPointData(blockID, CENTER, n, EXTERIOR, &
!                      axis, sim_smallX)
!              enddo
!           end if

           ! Compute the gas energy and set the gamma-values needed for the equation of 
           ! state.

           ! store the variables in the current zone via Grid put methods
           ! data is put stored one cell at a time with these calls to Grid_putData           

      elseif (f/=0.e0) then

        do i=max(-deltaCell + posCell(IAXIS),blkLimits(LOW,IAXIS)), &
               min(posCell(IAXIS) + deltaCell,blkLimits(HIGH,IAXIS))
          q1 = ( xCen(i-blkLimits(LOW,IAXIS)+1)-pos(1))**2
  
          do j=max(-deltaCell + posCell(JAXIS),blkLimits(LOW,JAXIS)), &
                 min(posCell(JAXIS) + deltaCell,blkLimits(HIGH,JAXIS))
            q2 = q1 + (yCen(j-blkLimits(LOW,JAXIS)+1)-pos(2))**2
  
            do k=max(-deltaCell + posCell(KAXIS),blkLimits(LOW,KAXIS)), &
                   min(posCell(KAXIS) + deltaCell,blkLimits(HIGH,KAXIS))
               q3 = q2 + (zCen(k-blkLimits(LOW,KAXIS)+1)-pos(3))**2
  
               r = sqrt(q3)
  
               if(r .lt. 2.E0*h) then

                 q4 = w(r,h)

                 rhoZone  = f * m * q4

                 if (blockID.eq.1) then
                   write(6,*) "RhoZone = ", rhoZone
                   write(6,*) "kernel = ", q4
                 endif

                 velZone(IAXIS) = vel(IAXIS)*rhoZone
                 velZone(JAXIS) = vel(JAXIS)*rhoZone
                 velZone(KAXIS) = vel(KAXIS)*rhoZone

                 eintZone = u*rhoZone
                 tempZone = T*rhoZone
                 presZone = P*rhoZone

                 solnData(DENS_VAR,i,j,k) = solnData(DENS_VAR,i,j,k) + rhoZone     
                 solnData(VELX_VAR,i,j,k) = solnData(VELX_VAR,i,j,k) + velZone(IAXIS)     
                 solnData(VELY_VAR,i,j,k) = solnData(VELY_VAR,i,j,k) + velZone(JAXIS)     
                 solnData(VELZ_VAR,i,j,k) = solnData(VELZ_VAR,i,j,k) + velZone(KAXIS)     
                 solnData(EINT_VAR,i,j,k) = solnData(EINT_VAR,i,j,k) + eintZone     
                 solnData(TEMP_VAR,i,j,k) = solnData(TEMP_VAR,i,j,k) + tempZone
                 solnData(PRES_VAR,i,j,k) = solnData(PRES_VAR,i,j,k) + presZone

                 xar36 = sim_xar36(n)*rhozone
                 xc12  = sim_xc12(n)*rhozone
                 xca40 = sim_xca40(n)*rhozone
                 xcr48 = sim_xcr48(n)*rhozone
                 xfe52 = sim_xfe52(n)*rhozone
                 xhe4  = sim_xhe4(n)*rhozone
                 xmg24 = sim_xmg24(n)*rhozone
                 xne20 = sim_xne20(n)*rhozone
                 xni56 = sim_xni56(n)*rhozone
                 xo16  = sim_xo16(n)*rhozone
                 xs32  = sim_xs32(n)*rhozone
                 xsi28 = sim_xsi28(n)*rhozone
                 xti44 = sim_xti44(n)*rhozone

                 solnData(AR36_SPEC,i,j,k) = solnData(AR36_SPEC,i,j,k) + xar36
                 solnData( C12_SPEC,i,j,k) = solnData( C12_SPEC,i,j,k) + xc12
                 solnData(CA40_SPEC,i,j,k) = solnData(CA40_SPEC,i,j,k) + xca40
                 solnData(CR48_SPEC,i,j,k) = solnData(CR48_SPEC,i,j,k) + xcr48
                 solnData(FE52_SPEC,i,j,k) = solnData(FE52_SPEC,i,j,k) + xfe52
                 solnData( HE4_SPEC,i,j,k) = solnData( HE4_SPEC,i,j,k) + xhe4
                 solnData(MG24_SPEC,i,j,k) = solnData(MG24_SPEC,i,j,k) + xmg24
                 solnData(NE20_SPEC,i,j,k) = solnData(NE20_SPEC,i,j,k) + xne20
                 solnData(NI56_SPEC,i,j,k) = solnData(NI56_SPEC,i,j,k) + xni56
                 solnData( O16_SPEC,i,j,k) = solnData( O16_SPEC,i,j,k) + xo16
                 solnData( S32_SPEC,i,j,k) = solnData( S32_SPEC,i,j,k) + xs32
                 solnData(SI28_SPEC,i,j,k) = solnData(SI28_SPEC,i,j,k) + xsi28
                 solnData(TI44_SPEC,i,j,k) = solnData(TI44_SPEC,i,j,k) + xti44

               endif             

             enddo

           enddo
 
         enddo
       endif
     endif
    
  enddo
  
!  call Logfile_stamp(myPE, "preparing Cell Data filename", "[Simulation_initBlock]")
!  write(outputFile,'(a,i3.3,a,i1.1,a)') 'Cell_values',blockID,'X',myPE,'.dat'

!  open(42+myPE,file=outputFile,status='unknown')

!  write(42+myPE,*) "#i  j  k         x         y         z         m"
   
  do k = blkLimits(LOW,KAXIS),blkLimits(HIGH,KAXIS)
     q3 = zCen(k-blkLimits(LOW,KAXIS)+1)
     
     do j = blkLimits(LOW,JAXIS),blkLimits(HIGH,JAXIS)
        q2 = yCen(j-blkLimits(LOW,JAXIS)+1)
        
        do i = blkLimits(LOW,IAXIS),blkLimits(HIGH,IAXIS)
          q1 = xCen(i-blkLimits(LOW,IAXIS)+1)
           
          if (solnData(DENS_VAR,i,j,k).eq.0.E0) then
             rhoZone = sim_rhoAmb
             eintZone = sim_eintAmb
             tempZone = max(sim_tempAmb,sim_smallT)
             presZone = sim_presAmb
             solnData(DENS_VAR,i,j,k) = rhoZone     
             solnData(TEMP_VAR,i,j,k) = tempZone
             solnData(PRES_VAR,i,j,k) = presZone
             eosMode =MODE_DENS_TEMP
          else
             rhoZone = solnData(DENS_VAR,i,j,k)
             eintZone = solnData(EINT_VAR,i,j,k) / rhoZone
             tempZone = max(solnData(TEMP_VAR,i,j,k) / rhoZone,sim_smallT)
             presZone = solnData(PRES_VAR,i,j,k) / rhoZone

             eosMode = MODE_DENS_EI
             solnData(VELX_VAR,i,j,k) = solnData(VELX_VAR,i,j,k) / rhoZone     
             solnData(VELY_VAR,i,j,k) = solnData(VELY_VAR,i,j,k) / rhoZone     
             solnData(VELZ_VAR,i,j,k) = solnData(VELZ_VAR,i,j,k) / rhoZone 
          endif
          
!          write(42+myPE,'(3(i3.2),3(ES10.3),ES14.7)') i,j,k,q1,q2,q3,rhoZone

          solnData(AR36_SPEC,i,j,k) = max(solnData(AR36_SPEC,i,j,k)/rhoZone, sim_smallx) 
          solnData( C12_SPEC,i,j,k) = max(solnData( C12_SPEC,i,j,k)/rhoZone, sim_smallx)
          solnData(CA40_SPEC,i,j,k) = max(solnData(CA40_SPEC,i,j,k)/rhoZone, sim_smallx)
          solnData(CR48_SPEC,i,j,k) = max(solnData(CR48_SPEC,i,j,k)/rhoZone, sim_smallx)
          solnData(FE52_SPEC,i,j,k) = max(solnData(FE52_SPEC,i,j,k)/rhoZone, sim_smallx)
          solnData( HE4_SPEC,i,j,k) = max(solnData( HE4_SPEC,i,j,k)/rhoZone, sim_smallx)
          solnData(MG24_SPEC,i,j,k) = max(solnData(MG24_SPEC,i,j,k)/rhoZone, sim_smallx)
          solnData(NE20_SPEC,i,j,k) = max(solnData(NE20_SPEC,i,j,k)/rhoZone, sim_smallx)
          solnData(NI56_SPEC,i,j,k) = max(solnData(NI56_SPEC,i,j,k)/rhoZone, sim_smallx)
          solnData( O16_SPEC,i,j,k) = max(solnData( O16_SPEC,i,j,k)/rhoZone, sim_smallx)
          solnData( S32_SPEC,i,j,k) = max(solnData( S32_SPEC,i,j,k)/rhoZone, sim_smallx)
          solnData(SI28_SPEC,i,j,k) = max(solnData(SI28_SPEC,i,j,k)/rhoZone, sim_smallx)
          solnData(TI44_SPEC,i,j,k) = max(solnData(TI44_SPEC,i,j,k)/rhoZone, sim_smallx)

          q4 = 0.
          do n=SPECIES_BEGIN,SPECIES_END
             q4 = q4 + solnData(n,i,j,k)
          enddo

          solnData(AR36_SPEC,i,j,k) = solnData(AR36_SPEC,i,j,k) / q4
          solnData( C12_SPEC,i,j,k) = solnData( C12_SPEC,i,j,k) / q4
          solnData(CA40_SPEC,i,j,k) = solnData(CA40_SPEC,i,j,k) / q4
          solnData(CR48_SPEC,i,j,k) = solnData(CR48_SPEC,i,j,k) / q4
          solnData(FE52_SPEC,i,j,k) = solnData(FE52_SPEC,i,j,k) / q4
          solnData( HE4_SPEC,i,j,k) = solnData( HE4_SPEC,i,j,k) / q4
          solnData(MG24_SPEC,i,j,k) = solnData(MG24_SPEC,i,j,k) / q4
          solnData(NE20_SPEC,i,j,k) = solnData(NE20_SPEC,i,j,k) / q4
          solnData(NI56_SPEC,i,j,k) = solnData(NI56_SPEC,i,j,k) / q4
          solnData( O16_SPEC,i,j,k) = solnData( O16_SPEC,i,j,k) / q4
          solnData( S32_SPEC,i,j,k) = solnData( S32_SPEC,i,j,k) / q4
          solnData(SI28_SPEC,i,j,k) = solnData(SI28_SPEC,i,j,k) / q4
          solnData(TI44_SPEC,i,j,k) = solnData(TI44_SPEC,i,j,k) / q4

          massFraction(AR36_SPEC) = solnData(AR36_SPEC,i,j,k)
          massFraction( C12_SPEC) = solnData( C12_SPEC,i,j,k)
          massFraction(CA40_SPEC) = solnData(CA40_SPEC,i,j,k)
          massFraction(CR48_SPEC) = solnData(CR48_SPEC,i,j,k)
          massFraction(FE52_SPEC) = solnData(FE52_SPEC,i,j,k)
          massFraction( HE4_SPEC) = solnData( HE4_SPEC,i,j,k)
          massFraction(MG24_SPEC) = solnData(MG24_SPEC,i,j,k)
          massFraction(NE20_SPEC) = solnData(NE20_SPEC,i,j,k)
          massFraction(NI56_SPEC) = solnData(NI56_SPEC,i,j,k)
          massFraction( O16_SPEC) = solnData( O16_SPEC,i,j,k)
          massFraction( S32_SPEC) = solnData( S32_SPEC,i,j,k)
          massFraction(SI28_SPEC) = solnData(SI28_SPEC,i,j,k)
          massFraction(TI44_SPEC) = solnData(TI44_SPEC,i,j,k)

          eosData(EOS_DENS) = rhoZone
          eosData(EOS_EINT) = eintZone
          eosData(EOS_TEMP) = tempZone
          eosData(EOS_PRES) = presZone

          call Eos(eosmode, 1, eosData, massFraction)

          if ( rhoZone /=  eosData(EOS_DENS) ) stop "Density was updated - things need to be renormalised"
          eintZone = eosData(EOS_EINT)
          presZone = eosData(EOS_PRES)
          tempZone = eosData(EOS_TEMP)
          gammZone    = eosData(EOS_GAMC)

!          if (((blockID.eq.1).and.(i.eq.1)).and.((j.eq.1).and.(k.eq.1))) &
!              write(6,*) "gamma = ", gammZone

          solnData(EINT_VAR,i,j,k) = eintZone

          ekinZone = 0.5E0*( solnData(VELX_VAR,i,j,k)*solnData(VELX_VAR,i,j,k) + &
                             solnData(VELY_VAR,i,j,k)*solnData(VELY_VAR,i,j,k) + &
                             solnData(VELZ_VAR,i,j,k)*solnData(VELZ_VAR,i,j,k) )
          enerZone = ekinZone + eintZone

          solnData(PRES_VAR,i,j,k) = presZone
          solnData(TEMP_VAR,i,j,k) = tempZone   
          solnData(ENER_VAR,i,j,k) = enerZone
          solnData(GAMC_VAR,i,j,k) = gammZone
          solnData(GAME_VAR,i,j,k) = gammZone
      
        enddo
     enddo
  enddo

!  close(42+myPE)

!! Cleanup!  Must deallocate arrays
  deallocate(xCen)
  deallocate(yCen)
  deallocate(zCen)

  call Grid_releaseBlkPtr(blockID, solnData)


 
  return
end subroutine Simulation_initBlock




!
!
!

subroutine findFact(xP,yP,zP,xo,yo,zo,hP,f,L, d,BlockID,myPE)

! ==================================================================== c
!   x,y,z = particle's position
!   xo,yo,zo = position of one cell center (usually the inner(i,j,k) cell
!   h     = particle's smoothing length
!   f     = particle's mass conservation factor
!   L     = uniform grid spacing
!   dim   = dimension = 1, 2, or 3
! ==================================================================== c
!
!   DECLARATIONS
! 
! ==================================================================== c

  implicit none

  real :: xP,yP,zP,x,y,z,xo,yo,zo,hP,h,f,L

  real :: w,r,mTot, q1, q2, q3

  integer i, j, k, n, d, BlockID, myPE

! ==================================================================== c
!
!   INITIALIZATIONS
!
! ==================================================================== c

  mTot = 0.E0
! ==================================================================== c
!
!   Renormalization:
!                    Make the grid have unit spacing
! ==================================================================== c

  x = (xP-xo)/L
  y = (yP-yo)/L
  z = (zP-zo)/L
  h = hP/L

! ==================================================================== c
!
!   Centering:
!              Translate particle so it is in the cell centered
!              at (0,0,0)
! ==================================================================== c

  x = mod(x,1.E0)
  if (x.gt.0.5E0) x = x - 1.E0

  y = mod(y,1.E0)
  if (y.gt.0.5E0) y = y - 1.E0

  z = mod(z,1.E0)
  if (z.gt.0.5E0) z = z - 1.E0

! ==================================================================== c
!
!   Determine bounds: 
!                     We will only look at cells within 2h of the 
!                     central cell's edge.
! ==================================================================== c

  d = int(2.E0*h + 0.5E0)

! ==================================================================== c
!
!   Loop over cells and if distance between cell and particle is less
!   than 2h, then determine cell's density and add to total mass.
!
!   Note: since h was rescaled, w is a factor of L**3 higher but when
!         multiplying by the volume of the cell, this is canceled out.      
!
! ==================================================================== c

  do k=-d, d
    q3=(dble(k)-z)**2

    do j=-d,d
      q2=q3+(dble(j)-y)**2

      do i=-d,d
        q1=q2+(dble(i)-x)**2

        r = sqrt(q1)

        if(r .lt. 2.E0*h) mTot = mTot + w(r,h)

      enddo

     enddo
 
  enddo

! ==================================================================== c
!
!   Find the factor f = m / mtot
!
! ==================================================================== c
  if (mTot.ne.0.E0) f = 1.E0/mTot

! ==================================================================== c

  return
end subroutine findFact
!
!
!
! ==================================================================== c
! ==================================================================== c
! ==================================================================== c


real function w(r,h)

  implicit none
  real :: r, h, v

  v = r/h

  w = 3.1415927410125732e0*h*h*h
  w = 1.0e0/w

  if (v.lt.1.e0) then
     w = w*(1-1.5e0*v*v+0.75e0*v*v*v)
     return
  else if (v.lt.2.0e0) then
     w = w*0.25e0*(2.0e0-v)**3
     return
  else
     w = 0.D0
  endif
 
  return
end function w

! ==================================================================== c










