!!****if* source/Simulation/SimulationMain/CheckEOS/Simulation_initBlock
!!
!! NAME
!!
!!  Simulation_initBlock
!!
!!
!! SYNOPSIS
!!
!!  Simulation_initBlock(integer(IN) :: blockID, 
!!                       integer(IN) :: myPE)
!!
!!
!!
!! DESCRIPTION
!!
!!  Initializes fluid data (density, pressure, velocity, etc.) for
!!  a specified block.  This version uses particle sph data to set up
!!  the block.
!!
!!
!! ARGUMENTS
!!
!!  blockID -           the number of the block to update
!!  myPE -             my processor number
!!
!! PARAMETERS
!!    ???
!!
!! GLOBAL VARIABLES
!!
!!  blkLow(3), blkHigh(3)
!! 
!!  rhoZone, velZone(3), presZone, eintZone, enerZone, ekinZone
!! 
!!***

subroutine Simulation_initBlock(blockID, myPE)

  use Simulation_data

  use Grid_interface, ONLY : Grid_getBlkIndexLimits, Grid_getCellCoords, &
                             Grid_getPointData, Grid_putPointData, &
                             Grid_getBlkPtr, Grid_releaseBlkPtr

  use Eos_interface, ONLY : Eos

  use Logfile_interface, ONLY : Logfile_stamp
  use tree, ONLY : lrefine, lrefine_max

  use PhysicalConstants_data, ONLY : pc_arrayUnit, pc_baseUnitLength, pc_baseUnitMass
  use PhysicalConstants_interface, ONLY : PhysicalConstants_init, PhysicalConstants_get

  implicit none

#include "constants.h"
#include "Flash.h"
#include "Eos.h"
  
  integer, intent(in) :: blockID
  integer, intent(in) :: myPE

! Dummy indices / values
  integer :: i, j, k

! Grid values
  real,allocatable, dimension(:) :: xCen, yCen, zCen
  integer :: sizeX,sizeY,sizeZ
  integer, dimension(2,MDIM) :: blkLimits, blkLimitsGC
  real, dimension(MDIM) :: blkLow, blkHigh, q
  real :: distance

! Values to be put in cell  
  real :: rhoZone,  presZone, eintZone, enerZone, ekinZone, tempZone, gammZone
  real, dimension(:,:,:,:),pointer :: solnData
  
! Variables for Eos
  real, dimension(EOS_NUM) :: eosData

  logical :: gcell = .true.
  integer :: istat 

  ! get the integer index information for the current block
  call Grid_getBlkIndexLimits(blockId,blkLimits,blkLimitsGC)
  
  ! Determine size of interior of block
  sizeX = blkLimitsGC(HIGH,IAXIS) - blkLimitsGC(LOW,IAXIS) + 1
  sizeY = blkLimitsGC(HIGH,JAXIS) - blkLimitsGC(LOW,JAXIS) + 1
  sizeZ = blkLimitsGC(HIGH,KAXIS) - blkLimitsGC(LOW,KAXIS) + 1

  ! allocate memory for iCen variables
  allocate(xCen(sizeX),stat=istat)
  allocate(yCen(sizeY),stat=istat)
  allocate(zCen(sizeZ),stat=istat)

  ! Initialize
  xCen = 0.0E0
  yCen = 0.0E0
  zCen = 0.0E0

  ! Get interior cell coordinates
  if (NDIM == 3) &
    call Grid_getCellCoords(KAXIS, blockId, CENTER ,gcell, zCen, sizeZ)

  if (NDIM >= 2) &
    call Grid_getCellCoords(JAXIS, blockId, CENTER,gcell, yCen, sizeY)

  call Grid_getCellCoords(IAXIS, blockId, CENTER,gcell, xCen, sizeX)
 
  ! Get lower and upper coordinates
  blkLow(1)   = xCen(1)
  blkHigh(1) = xCen(sizeX)

  blkLow(2)   = yCen(1)
  blkHigh(2) = yCen(sizeY)

  blkLow(3)   = zCen(1)
  blkHigh(3) = zCen(sizeZ)

  ! Get pointer to unk
  call Grid_getBlkPtr(blockId, solnData)

  ! Take note of some confusion:
  !   - Cell positions are ONLY found for the interior (not including GC)
  !   - solnData points to all cells INCLUDING GC
  !   - posCell will have to use the GC grid if it will later be used with solnData
  !      - thus for comparisons between posCell and iCen, iCen must have the same
  !        index as posCell minus blkLimits(DIRECTION) + 1

!------------------------------------------------------------------------------
! Loop over all particle positions. If particle's smoothing length is within the
! the block, then it's attributes are applied to its neighbouring cells, subject to
! the particle's smoothing kernel.

  do k = blkLimits(LOW,KAXIS),blkLimits(HIGH,KAXIS)
     do j = blkLimits(LOW,JAXIS),blkLimits(HIGH,JAXIS)
        do i = blkLimits(LOW,IAXIS),blkLimits(HIGH,IAXIS)

           
          eosData(EOS_DENS) = sim_rhoAmb
          eosData(EOS_TEMP) = sim_tempAmb

          call Eos(MODE_DENS_TEMP, 1, eosData)

          rhoZone =  eosData(EOS_DENS)
          eintZone = eosData(EOS_EINT)
          presZone = eosData(EOS_PRES)
          tempZone = eosData(EOS_TEMP)
          gammZone    = eosData(EOS_GAMC)

          solnData(DENS_VAR,i,j,k) = rhoZone
          solnData(EINT_VAR,i,j,k) = eintZone

          enerZone = eintZone

          solnData(PRES_VAR,i,j,k) = presZone
          solnData(TEMP_VAR,i,j,k) = tempZone   
          solnData(ENER_VAR,i,j,k) = enerZone
          solnData(GAMC_VAR,i,j,k) = gammZone
          solnData(GAME_VAR,i,j,k) = gammZone
      
        enddo
     enddo
  enddo

!  close(42)

!! Cleanup!  Must deallocate arrays
  deallocate(xCen)
  deallocate(yCen)
  deallocate(zCen)

  call Grid_releaseBlkPtr(blockID, solnData)

  return
end subroutine Simulation_initBlock

